#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2010 Gauvain Pocentek <gauvain.pocentek@objectif-libre.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import dbus
import dbus.service
from dbus.mainloop.glib import DBusGMainLoop
import gobject
import simplejson as json

import devices
import transactions
from VideoEnPoche.dal import Session, Movie

session = Session()

devices_handled = False

def convert_bytes(bytes):
    bytes = float(bytes)
    if bytes >= 1099511627776:
        terabytes = bytes / 1099511627776
        size = '%.2fT' % terabytes
    elif bytes >= 1073741824:
        gigabytes = bytes / 1073741824
        size = '%.2fG' % gigabytes
    elif bytes >= 1048576:
        megabytes = bytes / 1048576
        size = '%.2fM' % megabytes
    elif bytes >= 1024:
        kilobytes = bytes / 1024
        size = '%.2fK' % kilobytes
    else:
        size = '%.2fb' % bytes
    return size

class Bus:
    devices_dict = {}
    transactions_dict = {}

    def __init__ (self):
        DBusGMainLoop (set_as_default=True)

        self.system_bus = dbus.SystemBus()

        self.name = dbus.service.BusName ('info.videoenpoche.server', self.system_bus)
        self.service = Service (self, '/Devices')

        # start the UDisks service
        self.system_bus.get_object('org.freedesktop.UDisks', '/org/freedesktop/UDisks')
        # define signal callbacks
        self.system_bus.add_signal_receiver (self._device_added, 'DeviceAdded',
                                 'org.freedesktop.UDisks')
        self.system_bus.add_signal_receiver (self._device_removed, 'DeviceRemoved',
                                 'org.freedesktop.UDisks')
        self.system_bus.add_signal_receiver (self._device_changed, 'DeviceChanged',
                                 'org.freedesktop.UDisks')

    def run (self):
        gobject.MainLoop().run()

    def _device_added (self, obj):
        global devices_handled
        if not devices_handled:
            return
        proxy = self.system_bus.get_object ('org.freedesktop.UDisks', obj)
        props = dbus.Interface (proxy, 'org.freedesktop.DBus.Properties')
        iface = dbus.Interface (proxy, 'org.freedesktop.UDisks.Device')

        # IdType is defined if we have a filesystem
        if props.Get ('org.freedesktop.UDisks.devices', 'IdType'):
            obj_id = obj.split ('/')[-1]
            label = ''
            vendor = props.Get ('org.freedesktop.UDisks.devices', 'DriveVendor')
            model = props.Get ('org.freedesktop.UDisks.devices', 'DriveModel')
            size = props.Get ('org.freedesktop.UDisks.devices', 'DeviceSize')
            size_str = convert_bytes (size)
            label = "%s %s (%s)"%(vendor, model, size_str)
            self.devices_dict[obj_id] = devices.Device (proxy, obj_id, label)

    def _device_removed (self, obj):
        global devices_handled
        if not devices_handled:
            return
        obj_id = obj.split ('/')[-1]
        try:
            self.devices_dict.pop (obj_id)
            self.transactions_dict.pop (obj_id)
        except:
            pass

    def _device_changed (self, obj):
        global devices_handled
        if not devices_handled:
            return
        proxy = self.system_bus.get_object ('org.freedesktop.UDisks', obj)
        props = dbus.Interface (proxy, 'org.freedesktop.DBus.Properties')
        iface = dbus.Interface (proxy, 'org.freedesktop.UDisks.Device')
        if props.Get ('org.freedesktop.UDisks.devices', 'DeviceIsMounted') == False:
            obj_id = obj.split ('/')[-1]
            try:
                self.devices_dict.pop (obj_id)
                self.transactions_dict.pop (obj_id)
            except:
                pass

class Service (dbus.service.Object):
    def __init__ (self, bus, path):
        dbus.service.Object.__init__ (self, bus.system_bus, path)
        self.devices = bus.devices_dict
        self.transactions = bus.transactions_dict

    @dbus.service.method ('info.videoenpoche.server', out_signature='a{ss}')
    def GetDevices (self):
        ret = {}
        for k in self.devices:
            ret[k] = json.dumps (self.devices[k].prep_for_json (), skipkeys=True)
        return ret

    @dbus.service.method ('info.videoenpoche.server', out_signature='a{ss}')
    def GetAvailableDevices (self):
        ret = {}
        for k in self.devices:
            if not self.devices[k].copy_in_progress ():
                ret[k] = json.dumps (self.devices[k].prep_for_json (), skipkeys=True)
        return ret

    @dbus.service.method ('info.videoenpoche.server', in_signature='s', out_signature='s')
    def RemoveDevice (self, device):
        try:
            self.devices[device].umount ()
            self.devices.pop (device)
            return 'true'
        except:
            return 'false'

    @dbus.service.method ('info.videoenpoche.server', in_signature='sas')
    def InitTransaction (self, device_id, movies_id):
        device = self.devices[device_id]
        movies_dict = {}
        for i in movies_id:
            movies_dict[i] = session.query(Movie).get(i)
        if not self.transactions.has_key (device_id):
            self.transactions[device_id] = transactions.Transaction (device, movies_dict)

    @dbus.service.method ('info.videoenpoche.server', in_signature='s')
    def ConcludeTransaction (self, device_id):
        if self.transactions.has_key (device_id):
            self.transactions[device_id].conclude ()
            self.transactions.pop (device_id)

    @dbus.service.method ('info.videoenpoche.server', in_signature='s')
    def CancelTransaction (self, device_id):
        self.transactions[device_id].cancel ()

    @dbus.service.method ('info.videoenpoche.server', in_signature='s')
    def AckCancel (self, device_id):
        self.transactions[device_id].ack_cancel ()
        self.transactions.pop (device_id)

    @dbus.service.method ('info.videoenpoche.server', in_signature='s')
    def SetDevicesHandled (self, b):
        global devices_handled
        if b == "1":
            devices_handled = True
        else:
            devices_handled = False

    @dbus.service.method ('info.videoenpoche.server', out_signature='b')
    def GetDevicesHandled (self):
        return devices_handled
