#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2010 Gauvain Pocentek <gauvain.pocentek@objectif-libre.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import dbus
import gio
from threading import Thread
from subprocess import *

from errors import *

IFACE_STR = 'org.freedesktop.UDisks.devices'

class Device:
    def __init__ (self, proxy, obj_id, label):
        self.id = obj_id
        self.iface = dbus.Interface (proxy, 'org.freedesktop.UDisks.Device')
        self.props = dbus.Interface (proxy, "org.freedesktop.DBus.Properties")
        self.devicefile = self.props.Get (IFACE_STR, 'DeviceFile')
        self.devicesize = self.props.Get (IFACE_STR, 'DeviceSize') # bytes
        self.label = label
        self.usable = 1

        self.mount ()
        # compute the available disk space
        s = os.statvfs (self.mountpoint)
        self.free_space = s.f_bsize * s.f_bavail

        self.global_status = 0
        self.copy_status = {}
        self.copy_dest = {}
        self.cancellables = {}
        self.counter = {} # how many time did we try to copy a file

        self.is_cancelled = False

        # test if we can write on the device
        try:
            testfile = os.path.join (self.mountpoint, 'videoenpoche-testfile')
            open (testfile, "w").close ()
            os.unlink (testfile)
        except:
            self.usable = 0

    def mount (self):
        if self.props.Get (IFACE_STR, 'DeviceIsMounted'):
            self.mountpoint = self.props.Get (IFACE_STR, 'DeviceMountPaths')[0]
            return

        try:
            self.mountpoint = self.iface.FilesystemMount ('auto', dbus.Array([], 's'))
        except:
            raise MountFailedError ("Mount failed for device %s"%self.devicefile)

    def umount (self):
        if not self.props.Get (IFACE_STR, 'DeviceIsMounted'):
            return

        self.iface.FilesystemUnmount (dbus.Array([], 's'))

    def _copy_progress (self, current, total, user_data):
        if self.global_status == "cancel":
            return
        self.copy_status[user_data] = int ((current*100) / total)

        keys = self.copy_status.keys ()
        total = 0
        for k in keys:
            try:
                total += self.copy_status[k]
            except:
                pass
        self.global_status = int (total/len (keys))

    def _validate_copy (self, src):
        self.copy_status[src] = 'checking'
        dst = self.copy_dest[src]

        f = Popen (['md5sum', src], stdout=PIPE)
        src_sum = f.communicate ()[0].split ()[0]
        f = Popen (['md5sum', dst.get_path ()], stdout=PIPE)
        dst_sum = f.communicate ()[0].split ()[0]

        if (src_sum != dst_sum):
            return False
        return True

    def _sync_thread (self):
        f = Popen (['sync'])
        f.wait ()
        self.global_status = 0

    def _sync (self, threaded=True):
        if not threaded:
            self._sync_thread ()
        else:
            thread = Thread (target=self._sync_thread)
            thread.start ()

    def delete_dest (self, src):
        self.copy_dest[src].delete ()

    def renew_copy (self, src):
        self.counter[src] += 1
        self.delete_dest (src)
        self.copy (src)

    def multiple_copy (self, srcs):
        self.srcs = srcs

        for src in self.srcs:
            self.copy_status[src] = 0
            self.counter[src] = 0

        thread = Thread (target=self._multiple_copy_thread)
        thread.start ()

    def _multiple_copy_thread (self):
        for src in self.srcs:
            self.copy (src)
            if self.is_cancelled:
                return

        self.global_status = "syncing"
        self._sync (threaded=False)

        self.global_status = "validating"
        for src in self.srcs:
            ret = self._validate_copy (src)
            if not ret:
                self.renew_copy (src)

        self.global_status = "OK"

    def copy (self, src, dest=None):
        if dest:
            dest_path = os.path.join (self.mountpoint, dest)
        else:
            dest_path = os.path.join (self.mountpoint, os.path.basename (src))

        src_file = gio.File (path=src)
        dst_file = gio.File (path=dest_path)

        self.cancellables[src] = gio.Cancellable ()
        self.copy_status[src] = 0
        self.copy_dest[src] = dst_file
        self.counter[src] = 0
        src_file.copy (dst_file,
                       progress_callback=self._copy_progress,
                       flags=gio.FILE_COPY_OVERWRITE,
                       cancellable=self.cancellables[src],
                       user_data=src)

    def copy_in_progress (self):
        if len (self.copy_status) == 0:
            return False
        return True

    def cancel_copy (self, src):
        self.cancellables[src].cancel ()

    def cancel_transaction (self):
        self.is_cancelled = True
        self.global_status = "cancel"

    def ack_cancel (self):
        for src in self.copy_status.keys():
            self.copy_status.pop (src)
            self.cancel_copy (src)
            self.copy_dest[src].delete ()
            self.copy_dest.pop (src)
        self._sync ()
        self.is_cancelled = False

    def prep_for_json (self):
        a = {}
        a['global_status'] = self.global_status
        a['id'] = self.id
        a['status'] = self.copy_status
        a['label'] = self.label
        a['size'] = self.devicesize
        a['freespace'] = self.free_space
        a['usable'] = self.usable

        return a
